//-----------------------------------------------------------
//    matrix_and_vector_output.h
//-----------------------------------------------------------
#ifndef SYKES_NUMERIC_MATRIX_AND_VECTOR_IO
#define SYKES_NUMERIC_MATRIX_AND_VECTOR_IO

#include <iostream>
#include <sstream>
#include <type_traits>
#include "matrix_and_vector_base.h"

namespace sykes{ namespace numeric{
    
    template<typename Matrix>
    std::ostream& operator<<(
        typename std::enable_if<
            std::is_base_of<
                sykes::numeric::matrix_base, Matrix
            >::value, std::ostream&
        >::type _ost,
        Matrix const& _mat)
    {
        typedef typename Matrix::size_type size_type;
        size_type size1 = _mat.size1(), size2 = _mat.size2();
        
        std::ostringstream osst;
        osst.flags(_ost.flags ());
        osst.imbue(_ost.getloc ());
        osst.precision(_ost.precision ());
        
        osst << "[" << size1 << "][" << size2 << "]";
        osst << "{";
        for(size_type i = 0; i < size1; ++i){
            osst << "{";
            for(size_type j = 0; j < size2; ++j){
                osst << _mat(i, j);
                if(j != size2 - 1) osst << ",";
            }
            osst << "}";
            if(i != size1 - 1) osst << ",";
        }
        osst << "}";
        
        return _ost << osst.str().c_str();
    }
    
    template<typename Vector>
    std::ostream& operator<<(
        typename std::enable_if<
            std::is_base_of<
                sykes::numeric::vector_base, Vector
            >::value, std::ostream&
        >::type _ost,
        Vector const& _vec)
    {
        typedef typename Vector::size_type size_type;
        size_type size1 = _vec.size();
        
        std::ostringstream osst;
        osst.flags(_ost.flags ());
        osst.imbue(_ost.getloc ());
        osst.precision(_ost.precision ());
        
        osst << "[" << size1 << "]";
        osst << "{";
        for(size_type i = 0; i < size1; ++i){
            osst << _vec(i);
            if(i != size1 - 1) osst << ",";
        }
        osst << "}";
        
        return _ost << osst.str().c_str();
    }
}}//---- namespace numeric sykes
#endif

