//-----------------------------------------------------------
//    reverse_iterator.h
//-----------------------------------------------------------
#ifndef SYKES_NUMERIC_REVERSE_ITERATOR
#define SYKES_NUMERIC_REVERSE_ITERATOR

#include <iterator>

namespace sykes{ namespace numeric{
	//-----------------------------------------------------------
    //    class interval_iterator
    //-----------------------------------------------------------
	template<typename Iterator>
	class reverse_iterator : public std::reverse_iterator<Iterator>
	{
	public:
		typedef reverse_iterator self_type;
		typedef std::reverse_iterator<Iterator> base_iterator_type;
		typedef typename Iterator::dual_iterator dual_iterator;
		typedef typename Iterator::dual_reverse_iterator dual_reverse_iterator;
		
		//---- constructor
		inline reverse_iterator()
			: base_iterator_type() {}
		
		inline reverse_iterator(Iterator _pos)
			: base_iterator_type(_pos) {}
		
		template<typename Ite>
		inline reverse_iterator(reverse_iterator<Ite> const& _other)
			: base_iterator_type(_other.base()) {}
		
		//---- begin end
		inline dual_iterator begin() const
		{
			Iterator tmp = base_iterator_type::base();
			--tmp;
			return tmp.begin();
		}
		
		inline dual_iterator end() const
		{
			Iterator tmp = base_iterator_type::base();
			--tmp;
			return tmp.end();
		}
		
		inline dual_reverse_iterator rbegin() const
		{
			Iterator tmp = base_iterator_type::base();
			--tmp;
			return tmp.rbegin();
		}
		
		inline dual_reverse_iterator rend() const
		{
			Iterator tmp = base_iterator_type::base();
			--tmp;
			return tmp.rend();
		}
	};//---- class reverse_iterator
}}//---- namespace numeric sykes
#endif

