//-----------------------------------------------------------
//    solver_CG.h
//-----------------------------------------------------------
#ifndef SYKES_SOLVER_CG
#define SYKES_SOLVER_CG

#include <stdexcept>
#include <algorithm>

namespace sykes{ namespace solver{
    //-----------------------------------------------------------
    //    conjugate_gradient_method
    //-----------------------------------------------------------
    template<class M, class V1, class V2>
    void conjugate_gradient_method(
        M const& mat,
        V1 const& vec,
        V2& ansv,
        double epsilon = 1.0E-6)
    {
        typedef M matrix_type;
        typedef V1 vector_type1;
        typedef V2 vector_type2;
        typedef typename M::size_type size_type;
        typedef typename M::value_type value_type;
        size_type size1 = mat.size1(), size2 = mat.size2(), sizev = vec.size();
        if((size1 != size2) || (size1 != sizev))
            throw std::runtime_error("sykes::solver::conjugate_gradient_method()");
        
        epsilon = fabs(epsilon);
        
        value_type err = 0.0, corr = 0.0,
            pre_err = 0.0, beta = 0.0;
        vector_type1 resv(vec), xv(size1), pv(resv);
        
        pre_err = inner_prod(resv, resv);
        size_type n1 = 0;
        while(1){
            vector_type1 vtmp1 = prod(mat, pv);
            value_type dtmp1 = inner_prod(pv, vtmp1);
            
            corr = pre_err / dtmp1;
            
            xv += pv * corr;
            resv -= vtmp1 * corr;
            
            err = inner_prod(resv, resv);
            
            if(err < epsilon) break;
            
            beta = err / pre_err;
            pre_err = err;
            pv = resv + beta * pv;
        }
    
        ansv = xv;
    }

}}//---- namespace
#endif

