#define BOOST_TEST_MODULE test_vector
#include "vector.h"
#include "matrix_and_vector_output.h"
#include <boost/test/included/unit_test.hpp>
#include <string>
#include <algorithm>
using namespace sykes::numeric;

BOOST_AUTO_TEST_CASE( construction )
{
	int size1 = 10;
	vector<double> v1(size1, 0);
	
	std::cout << v1 << std::endl;
	
	for(int i1 = 0; i1 < size1; ++i1){
		v1(i1) = i1 * 2 + 1;
	}
	
	std::cout << v1 << std::endl;
	
	vector<double> v2(v1), v3;
	
	v3 = v2;
	
	std::cout << v2 << std::endl;
}

BOOST_AUTO_TEST_CASE( resize )
{
	int size1 = 10;
	vector<int> v1(size1);
	for(int i1 = 0; i1 < size1; ++i1){
		v1(i1) = i1;
	}
	std::cout << v1 << std::endl;
	
	v1.resize(20, true, 10);
	
	BOOST_CHECK_EQUAL(v1.size(), 20);
	
	std::cout << "resized" << std::endl;
	std::cout << v1 << std::endl;
	
	v1.resize(5);
	
	BOOST_CHECK_EQUAL(v1.size(), 5);
	
	std::cout << "re resized" << std::endl;
	std::cout << v1 << std::endl;
	
	v1.resize(10, false, 9);
	BOOST_CHECK_EQUAL(v1.size(), 10);
	std::cout << "re re resized" << std::endl;
	std::cout << v1 << std::endl;
	
}


struct Element
{
	static int construction_count;
	static int destruction_count;
	
	Element(){++construction_count;}
	Element(Element const& _other){++construction_count;}
	~Element(){++destruction_count;}
};
int Element::construction_count = 0;
int Element::destruction_count = 0;

BOOST_AUTO_TEST_CASE( element_construction_and_destruction_count )
{
	{
		int size1 = 100;
		vector<Element> X(size1);
		vector<Element> Y(X), Z(0);
		Z = Y;
		Z.resize(10000);
	}
	std::cout << "construction count: " << Element::construction_count << std::endl;
	std::cout << "destruction count: " << Element::destruction_count << std::endl;
	BOOST_CHECK_EQUAL(Element::construction_count, Element::destruction_count);
}

BOOST_AUTO_TEST_CASE( size_test )
{
	const int size1 = 10;
	vector<double> v1(size1);
	BOOST_CHECK_EQUAL(v1.size(), size1);
	
	vector<double> v2(v1), v3;
	BOOST_CHECK_EQUAL(v2.size(), size1);
	
	v3 = v2;
	BOOST_CHECK_EQUAL(v3.size(), size1);
}

BOOST_AUTO_TEST_CASE( at_test )
{
	int size1 = 10;
	vector<int> v1(size1);
	for(int i1 = 0; i1 < size1; ++i1){
		v1(i1) = i1;
	}
	
	BOOST_CHECK_EQUAL(v1.at(5), 5);
	BOOST_CHECK_THROW(v1.at(10), std::exception );
}

BOOST_AUTO_TEST_CASE( iterator_test )
{
	const int size1 = 10;
	vector<int> v1(size1);
	for(int i1 = 0; i1 < size1; ++i1){
		v1(i1) = i1 * i1 + 2;
	}
	std::cout << v1 << std::endl;
	{
		auto it1 = v1.begin(), end1 = v1.end();
		while(it1 != end1){
			
			std::cout << *it1 << std::endl;
			++it1;
		}
	}
	
	{
		auto it1 = v1.rbegin(), end1 = v1.rend();
		while(it1 != end1){
			
			std::cout << *it1 << std::endl;
			++it1;
		}
	}
}

BOOST_AUTO_TEST_CASE( iterator_comparison )
{
	const int size1 = 10;
	vector<int> m1(size1);
	
	vector<int>::iterator it1 = m1.begin(), it2 = m1.begin();
	vector<int>::const_iterator cit1 = m1.begin(), cit2 = m1.begin();
	
	BOOST_CHECK(it1 == it2);
	BOOST_CHECK(it1 == cit1);
	BOOST_CHECK(cit1 == it1);
	BOOST_CHECK(cit1 == cit2);
	
}


BOOST_AUTO_TEST_CASE( container_comparison )
{
	int size1 = 10;
	vector<double> v1(size1, 0.0);
	
	for(int i1 = 0; i1 < size1; ++i1){
		v1(i1) = double(i1 * 2 + 1);
	}
	
	vector<double> v2(v1), v3;
	
	BOOST_CHECK_EQUAL(v1, v2);
	
	v2(0) += 1.0;
	BOOST_CHECK(v1 != v2);
	
	v3 = v2;
	BOOST_CHECK_EQUAL(v2, v3);
	
}


